"""
Extracts workflow relevant metadata from the NRAO archive based on Science Product Locator.
"""
import argparse
import json
import logging
import sys

import pendulum
import psycopg2 as pg
from pycapo import CapoConfig

from aat_wrest.observation_wrester import ObservationWrester
from aat_wrest.utilities import MDDBConnector, TIME_ZONE, PENDULUM_FORMAT

logger = logging.getLogger("aat_wrest")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class WrestWorkflowMetadata:
    """
    Class for extracting metadata required for workflow execution
    """
    def __init__(self, connection, spl: str):
        self.conn = connection
        self.spl = spl

    def wrest_standard_cal_info(self) -> json:
        """
        Given an execution block science product locator, returns the required metadata to run
        the standard calibration workflow
        :return:
        """

        sql = f"""
        SELECT ngas_fileset_id as filesetId,
         e.project_code as projectCode,
         p.title as title,
         e.starttime as startTime,
         (a.firstname || ' ' || a.lastname) as observer,
         telescope as telescope
         FROM execution_blocks e
            JOIN projects p on e.project_code = p.project_code
            JOIN authors a on p.project_code = a.project_code
         WHERE science_product_locator = %(spl)s AND a.is_pi = true
        """
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql, {"spl": self.spl})
            data = cursor.fetchall()
            make_json = json.dumps(
                {
                    "sdmId": data[0][0],
                    "projectCode": data[0][1],
                    "title": data[0][2],
                    "startTime": data[0][3],
                    "observer": data[0][4],
                    "telescope": data[0][5],
                    "created_at": str(
                        pendulum.now().in_timezone(TIME_ZONE).format(PENDULUM_FORMAT)
                    ),
                }
            )
        finally:
            self.conn.close()
        return make_json


def parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces-to-Archive Metadata Exchange",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "-sc",
        "--stdcals",
        nargs=1,
        action="store",
        required=False,
        help="Find workflow metadata for standard calibrations with provided product locator",
    )
    parser.add_argument(
        "-obs",
        "--observation",
        nargs=1,
        action="store",
        required=False,
        help="Find display metadata for observations by product locator"
    )
    return parser


def determine_wrester(connection, args: argparse.Namespace):
    if args.stdcals:
        data = WrestWorkflowMetadata(connection, args.stdcals[0]).wrest_standard_cal_info()
    if args.observation:
        data = ObservationWrester(connection, args.observation[0]).wrest_observation_info()

    print(data)


def main():
    args = parser().parse_args()
    connection = MDDBConnector()

    determine_wrester(connection, args)

